@@ -10,7 +10,7 @@ use swc_ecma_ast::*;
 use swc_ecma_utils::{
     ident::IdentLike, member_expr, quote_ident, quote_str, undefined, ExprFactory,
 };
-use swc_ecma_visit::{noop_fold_type, noop_visit_type, Fold, FoldWith, Node, Visit, VisitWith};
+use swc_ecma_visit::{noop_fold_type, noop_visit_type, Fold, FoldWith, Visit, VisitWith};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub(super) struct Loc {
@@ -1539,7 +1539,7 @@ struct LeapFinder {
 
 macro_rules! leap {
     ($name:ident,$T:ty) => {
-        fn $name(&mut self, _: &$T, _: &dyn Node) {
+        fn $name(&mut self, _: &$T) {
             self.found = true;
         }
     };
@@ -1549,9 +1549,9 @@ impl Visit for LeapFinder {
     noop_visit_type!();
 
     /// Ignored
-    fn visit_function(&mut self, _: &Function, _: &dyn Node) {}
+    fn visit_function(&mut self, _: &Function) {}
     /// Ignored
-    fn visit_arrow_expr(&mut self, _: &ArrowExpr, _: &dyn Node) {}
+    fn visit_arrow_expr(&mut self, _: &ArrowExpr) {}
 
     leap!(visit_yield_expr, YieldExpr);
     leap!(visit_break_stmt, BreakStmt);
@@ -1565,7 +1565,7 @@ where
     T: VisitWith<LeapFinder>,
 {
     let mut v = LeapFinder { found: false };
-    node.visit_with(&Invalid { span: DUMMY_SP } as _, &mut v);
+    node.visit_with(&mut v);
     v.found
 }
 
